/*----------------------------------------------------------------------------
 Copyright:      Radig Ulrich  mailto: mail@ulrichradig.de
 Author:         Radig Ulrich
 Remarks:        
 known Problems: none
 Version:        24.10.2007
 Description:    Webserver uvm.

 Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen der 
 GNU General Public License, wie von der Free Software Foundation verffentlicht, 
 weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder 
 (nach Ihrer Option) jeder spteren Version. 

 Die Verffentlichung dieses Programms erfolgt in der Hoffnung, 
 da es Ihnen von Nutzen sein wird, aber OHNE IRGENDEINE GARANTIE, 
 sogar ohne die implizite Garantie der MARKTREIFE oder der VERWENDBARKEIT 
 FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License. 

 Sie sollten eine Kopie der GNU General Public License zusammen mit diesem 
 Programm erhalten haben. 
 Falls nicht, schreiben Sie an die Free Software Foundation, 
 Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
----------------------------------------------------------------------------*/

#include <avr/io.h>
#include "config.h"
#include "usart.h"
#include "networkcard/enc28j60.h"
#include "networkcard/rtl8019.h"
#include "stack.h"
#include "timer.h"
#include "wol.h"
#include "httpd.h"
#include "cmd.h"
#include "telnetd.h"
#include "ntp.h"
#include "base64.h"
#include "http_get.h"
#include "lcd.h"
#include "udp_lcd.h"
#include "analog.h"
#include "camera/cam.h"
#include "camera/servo.h"
#include "sendmail.h"


//#define MES_DEBUG	usart_write 
//#define MES_DEBUG(...)	

// 1-Wire ------------------------------------------------------------------------------
#if USE_OW
#include "1-wire/ds18x20.h"
// Variable
uint16_t 	ow_array[MAXSENSORS];	// Speicherplatz fr 1-wire Sensorwerte


PROGMEM	uint8_t		DS18B20IDs[MAXSENSORS+1][OW_ROMCODE_SIZE] = {
							OW_ID_T01,	// 1. DS18B20
							OW_ID_Last };	// Endmarker

uint8_t auslesen = 0;

#endif
// 1-Wire -------------------------------------------------------------------------------



//Hier startet das Hauptprogramm

int main(void)
{  
	//Konfiguration der Ausgnge bzw. Eingnge
	//definition erfolgt in der config.h
	DDRA = OUTA;
	DDRC = OUTC;
	DDRD = OUTD;
	
    unsigned long a;
	#if USE_SERVO
		servo_init ();
	#endif //USE_SERVO
	
    usart_init(BAUDRATE); // setup the UART
	
	#if USE_ADC
		ADC_Init();
	#endif
	
	usart_write("\n\rSystem Ready\n\r");
    usart_write("Compiliert am "__DATE__" um "__TIME__"\r\n");
    usart_write("Compiliert mit GCC Version "__VERSION__"\r\n");
	for(a=0;a<1000000;a++){asm("nop");};

	//Applikationen starten
	stack_init();
	httpd_init();
	telnetd_init();
	
	//Spielerrei mit einem LCD
	#if USE_SER_LCD
	udp_lcd_init();
	lcd_init();
	lcd_clear();
	back_light = 1;
	lcd_print(0,0,"System Ready");
	#endif
	//Ethernetcard Interrupt enable
	ETH_INT_ENABLE;
	
	//Globale Interrupts einschalten
	sei(); 
	
	#if USE_CAM
		#if USE_SER_LCD
		lcd_print(1,0,"CAMERA INIT");
		#endif //USE_SER_LCD
	for(a=0;a<2000000;a++){asm("nop");};
	cam_init();
	max_bytes = cam_picture_store(CAM_RESELUTION);
		#if USE_SER_LCD
		back_light = 0;
		lcd_print(1,0,"CAMERA READY");
		#endif //USE_SER_LCD
	#endif //USE_CAM
	
	#if USE_NTP
        ntp_init();
        ntp_request();
	#endif //USE_NTP
	
	#if USE_WOL
        wol_init();
	#endif //USE_WOL
    
	#if USE_MAIL
        mail_client_init();
	#endif //USE_MAIL

	#if USE_OW 
	usart_write("1-Wire vorhanden\r\n");
	#endif

// Hier beginnt die Endlosschleife

	while(1)
	{
		#if USE_ADC
		ANALOG_ON;
		#endif
	    eth_get_data();
		
        //Terminalcommandos auswerten
		if (usart_status.usart_ready){
            usart_write("\r\n");
			if(extract_cmd(&usart_rx_buffer[0]))
			{
				usart_write("Ready\r\n\r\n");
			}
			else
			{
				usart_write("ERROR\r\n\r\n");
			}
			usart_status.usart_ready =0;
		}


	//RoBue -> Uhrzeit berechnen
	unsigned char hh = (time/3600)%24;
	unsigned char mm = (time/60)%60;
	unsigned char ss = time %60;

	
// 1-Wire --------------------------------------------------------------------------------	
	

	// 1 DS18B20 auslesen
	// RoBue Anmerkung:
	// - Sekunde 58 -> Sensor zum Messen und Berechnen auffordern
	// - delay vom 1000 ms (so lange braucht der Sensor ungef.)
	// - Sekunde 00 -> Sensor(en) auslesen
	// Ausgabe ueber UART und in ow_array

	#if USE_OW

		// Testausgabe UART:
		// usart_write("1Wire - Auslesen\r\n");		

		uint8_t i = 0;
		uint16_t TWert;
		uint8_t subzero, cel, cel_frac_bits;
		uint8_t tempID[OW_ROMCODE_SIZE];
		
		if ( ss == 58 ) {

		// RoBue Anmerkung:
		// Hiermit werden ALLE Sensoren zum Messen aufgefordert.
		// Aufforderung nur bestimmter Sensoren:
		// NULL -> tempID 
		DS18X20_start_meas( DS18X20_POWER_PARASITE, NULL );
		_delay_ms(1000);
		// Jetzt kann wieder ausgelesen werden
		auslesen = 1;

		}

		if ( ss == 00 ) {

		// Nur einmal auslesen
		if ( auslesen == 1 ) { 

		// (erste) ID ins RAM holen
		memcpy_P(tempID,DS18B20IDs[0],OW_ROMCODE_SIZE);	
				
		// Evtl. Test, ob 1-Wire-Geraet ein Temperatursensor ist
		// if ( tempID[0] == DS18S20_ID ) ...

						
			// RoBue Anmerkung:
			// Hiermit wird ein einzelner Sensor ausgelesen
			// und die Temperatur in ow_array abgelegt (8 Werte/Sensor).
			// Position 0-4 eig. ID, Pos. 5+6 ist die Temperatur, Pos. 7 CRC-Byte
			if ( DS18X20_read_meas( tempID, &subzero,&cel, &cel_frac_bits) == DS18X20_OK ) {
				TWert = DS18X20_temp_to_decicel(subzero, cel, cel_frac_bits);
				ow_array[i] = TWert;
				
				// Testausgabe UART:
				usart_write("%2i:%2i:%2i: Temperatur: %3i Grad\r\n",hh,mm,ss,ow_array[i]/10);
				
			}
			else {
				usart_write("\r\nCRC Error (lost connection?) \r\n");
				DS18X20_show_id_uart( tempID, OW_ROMCODE_SIZE );


			}

		}	// -> auslesen
			
		
		// nchste ID ins RAM holen
		// memcpy_P(tempID,DS18B20IDs[++i],OW_ROMCODE_SIZE);	

		auslesen = 0;

		}	// -> ss

	#endif

// 1-Wire --------------------------------------------------------------------------------

	
        //Wetterdaten empfangen (Testphase)
        #if GET_WEATHER
        http_request ();
        #endif
        
        //Empfang von Zeitinformationen
		#if USE_NTP
		if(!ntp_timer){
			ntp_timer = NTP_REFRESH;
			ntp_request();
		}
		#endif //USE_NTP
		
        //Versand von E-Mails
        #if USE_MAIL
        if (mail_enable == 1)
        {
            mail_enable = 0;
            mail_send();
        }
        #endif //USE_MAIL
        
        //Rechner im Netzwerk aufwecken
        #if USE_WOL
        if (wol_enable == 1)
        {
            wol_enable = 0;
            wol_request();
        }
        #endif //USE_WOL
           
		//USART Daten fr Telnetanwendung?
		telnetd_send_data();
    }
		
return(0);
}